// Some copyright should be here...

using UnrealBuildTool;
using System.IO;

public class ViveportSDK : ModuleRules
{
    private string ModulePath
    {
        get { return ModuleDirectory; }
    }

    private string ThirdPartyPath
    {
        get { return Path.GetFullPath(Path.Combine(ModulePath, "../../ThirdParty/")); }
    }

    public ViveportSDK(ReadOnlyTargetRules Target) : base(Target)
    {
        PublicIncludePaths.AddRange(
            new string[] {
                "ViveportSDK/Public"
                // ... add public include paths required here ...
            }
        );
        PrivateIncludePaths.AddRange(
            new string[] {
                "ViveportSDK/Private",
                Path.Combine(ThirdPartyPath, "Viveport", "Includes"),
                // ... add other private include paths required here ...
            }
        );
        PublicDependencyModuleNames.AddRange(
            new string[]
            {
                "Core",
                // ... add other public dependencies that you statically link with here ...
            }
        );
        PrivateDependencyModuleNames.AddRange(
            new string[]
            {
                "CoreUObject",
                "Engine",
                "Json",
                "JsonUtilities"
                // ... add private dependencies that you statically link with here ...    
            }
        );
        DynamicallyLoadedModuleNames.AddRange(
            new string[]
            {
                // ... add any modules that your module loads dynamically here ...
            }
        );

        LoadViveportLibrary(Target);
    }

    public bool LoadViveportLibrary(ReadOnlyTargetRules Target)
    {
        bool isLibrarySupported = false;

        if ((Target.Platform == UnrealTargetPlatform.Win64) || (Target.Platform == UnrealTargetPlatform.Win32))
        {
            isLibrarySupported = true;
            string platformString = Target.Platform == UnrealTargetPlatform.Win64 ? "Win64" : "Win32";
            platformString = Path.Combine("Libraries", platformString);
            string LibrariesPath = Path.Combine(ThirdPartyPath, "Viveport", platformString);

            PublicLibraryPaths.Add(LibrariesPath);
            PublicAdditionalLibraries.Add("viveport_api.lib");
            PublicAdditionalLibraries.Add("viveport_ext_api.lib");
            PublicDelayLoadDLLs.Add("viveport_api.dll");
            PublicDelayLoadDLLs.Add("viveport_ext_api.dll");
            RuntimeDependencies.Add(new RuntimeDependency(Path.Combine(LibrariesPath, "viveport_api.dll")));
            RuntimeDependencies.Add(new RuntimeDependency(Path.Combine(LibrariesPath, "viveport_ext_api.dll")));
        }
        else if (Target.Platform == UnrealTargetPlatform.Android)
        {
            string PluginPath = Utils.MakePathRelativeTo(ModuleDirectory, Target.RelativeEnginePath);
            System.Console.WriteLine("_______________________ Path " + PluginPath);
            AdditionalPropertiesForReceipt.Add(new ReceiptProperty("AndroidPlugin", Path.Combine(PluginPath, "ViveportSDK_APL.xml")));
            PublicAdditionalLibraries.Add(ThirdPartyPath + "Viveport/jniLibs/armeabi-v7a/libViveportSdkJni.so");
        }

        if (isLibrarySupported)
        {
            // Include path
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "Viveport", "Includes"));
        }

        Definitions.Add(string.Format("WITH_VIVEPORTSDK_BINDING={0}", isLibrarySupported ? 1 : 0));

        return isLibrarySupported;
    }
}
